/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
ldse@hendriklipka.de
*/

package de.hendriklipka.ldse.common;

import java.util.Date;
import java.io.PrintWriter;

import com.objectspace.jgl.BinaryPredicate;

/**
  Beschreibt ein einzelnes Ergebnis
*/
public class ResultDescriptor implements BinaryPredicate
{
  protected String _urlStr=null;
  protected int _size=0;
  protected Date _lastChanged=null;
  protected String _title=null;
  protected String _excerpt=null;
  protected String _type=null;
  protected double _relevance=0;
  
  public ResultDescriptor()
  {
  }
  
  public void setRelevance(double d)
  {
    _relevance=d;
  }
  
  public double getRelevance()
  {
    return _relevance;
  }

  public void setURL(String s)
  {
    _urlStr=s;
  }
  
  public String getURL()
  {
    return _urlStr;
  }
  
  public void setSize(int i)
  {
    _size=i;
  }
  
  public int getSize()
  {
    return _size;
  }
  
  public void setTitle(String s)
  {
    _title=s;
  }
  
  public String getTitle()
  {
    return _title;
  }
  
  public String getExcerpt()
  {
    return _excerpt;
  }
  
  public void setExcerpt(String s)
  {
    _excerpt=s;
  }
  
  public void setLastChange(String s)
  {
    try
    {
      long l=Long.parseLong(s);
      _lastChanged=new Date(l);
    }
    catch (Exception ex) {}
  }
  
  public void setLastChange(long l)
  {
    _lastChanged=new Date(l);
  }
  
  public void setLastChange(Date d)
  {
    _lastChanged=d;
  }
  
  public Date getLastChange()
  {
    return _lastChanged;
  }
  
  public void setType(String s)
  {
    _type=s;
  }
  
  public String getType()
  {
    return _type;
  }

  public int compareTo(Object o)
  {
    try
    {
      ResultDescriptor rd=(ResultDescriptor)o;
      if (rd._relevance==_relevance)
        return 0;
      if (rd._relevance<_relevance)
        return 1;
      if (rd._relevance>_relevance)
        return -1;
      return 0;
    }
    catch (Exception ex)
    {
      return 0;
    }
  }
  
  public boolean equals(Object o)
  {
    ResultDescriptor rd=(ResultDescriptor)o;
    return (rd._relevance==_relevance);
  }

  /**
    Ausgabe als XML-Knoten
  */
  public void writeAsXML(PrintWriter writer)
  {
    writer.println("<answer>");
    writer.println("<url>"+_urlStr+"</url>");
    writer.println("<relevance>"+_relevance+"</relevance>");
    writer.println("<title>"+_title+"</title>");
    writer.println("<excerpt>"+_excerpt+"</excerpt>");
    writer.println("<size>"+_size+"</size>");
    writer.println("<lastchange>"+_lastChanged.getTime()+"</lastchange>");
    writer.println("<type>"+_type+"</type>");
    writer.println("</answer>");
    writer.flush();
  }

  // Vergleichsfunktionen
  public static BinaryPredicate getPredicate()
  {
    return new ResultDescriptor();
  }
  
  public boolean execute(Object first, Object second)
  {
    ResultDescriptor rd1=(ResultDescriptor)first;
    ResultDescriptor rd2=(ResultDescriptor)second;
    return (rd1._relevance>rd2._relevance);
  }
  
  public int hashCode()
  {
    return new Double(_relevance).hashCode();
  }
}
