/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
ldse@hendriklipka.de
*/

package de.hendriklipka.ldse.server.data.document;

import java.io.*;

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.common.HTMLCommentFilterReader;
import de.hendriklipka.ldse.common.HTMLTagFilterReader;
import de.hendriklipka.ldse.server.data.document.Document;
import de.hendriklipka.ldse.server.data.document.Triplet;
import de.hendriklipka.ldse.server.data.URLInfo;

/**
  zerlegt ein HTML-Dokument in einzelne Worte
*/
public class HTMLDocument extends Document
{
  protected HTMLTagFilterReader _reader=null;
  protected String _word=null;
  protected String _title="";
  
  public boolean init()
  {
		try
		{
      _reader=new HTMLTagFilterReader(new HTMLCommentFilterReader(new BufferedReader(_urlInfo.getReaderForURL(_urlInfo._url))));
      return true;
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(Log.getWriter("document","error",0));
    }
    return false;
  }
  
  public boolean hasMoreWords()
  {
		// einlesen...
		// es wird immer ein ganzes Wort eingelesen und gemerkt
    if (null==_reader)
      return false;
    if (null!=_word)
      return true;
    try
    {
      int c;
      char ch;
      c=_reader.read();
      ch=(char)c;
      if (-1==c)
        return false;
      while (!Triplet.isValidChar(ch))
      {
        if (!_reader.ready())
          return false;
        c=_reader.read();
        ch=(char)c;
        if (-1==c)
          return false;
      }
      StringBuffer sb=new StringBuffer();
      while (Triplet.isValidChar(ch))
      {
        sb.append(ch);
        if (!_reader.ready())
          break;
        c=_reader.read();
        ch=(char)c;
        if (-1==c)
          break;
      }
      _word=sb.toString();
      if (0==_word.length())
        return false;
      return true;
    }
    catch (IOException ex)
    {
      ex.printStackTrace();
      return false;
    }
  }
  
  public String nextWord()
  {
    if (null==_word)
    {
      if (!hasMoreWords())
        return null;
    }
    String s=_word;
    _word=null;
    return s;
  }
  
  public void finish()
  {
    _docDesc._title=_reader.getTitle();
    try
    {
        if (null!=_reader)
      _reader.close();
    }
    catch (IOException ex) {}
  }  

  public String getDescriptiveWords()
  {
    return _reader.getKeywords()+" "+_reader.getAuthor()+" "+_reader.getDescription()+
            " "+_reader.getDublinCore()+" "+_reader+getTitle();
  }
}