/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
ldse@hendriklipka.de
*/

package de.hendriklipka.ldse.server.dataserver;

import java.net.*;
import java.io.*;
import java.util.*;

import org.w3c.dom.*;

import de.hendriklipka.ldse.server.data.document.*;
import de.hendriklipka.ldse.server.data.server.ServerDescriptor;
import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.common.Updateable;
import de.hendriklipka.ldse.common.StatisticWriter;

/**
  Nimmt Anfragen ueber Daten und deren Veraenderungen entgegen.
  Aendert die Daten in der Datenbank bzw. liefert Ergebnisse zurueck.
  Anfragen koennen ueber einen Socket gestellt werden.
*/
public class DataServer extends java.lang.Thread implements de.hendriklipka.ldse.common.Updateable, de.hendriklipka.ldse.common.StatisticWriter
{
  // Datenbank-Connection
  protected DBConnection _con=null;
  
  // Konfiguration
  private boolean _isAdmin=false;
  private String _passwd="passwd";
  protected String _server=null;
  protected int _port=7623;
  protected boolean _reconfig=false;
  protected String _parentServer=null;
  protected int _parentPort=7623;
  protected int _id=-1;
  protected String _name="local";
  protected int _queryPort=-1;
  
  // wurden Daten veraendert?
  protected boolean _dataChanged=false;
  protected boolean _subServerChanged=false;
  
  //ToDo - wieder auf 0 setzen...
  protected int _threadCount=1;
  protected Object _countLock=new Object();

  protected Updateable _updateMaster=null;
  protected StatisticWriter _statisticMaster=null;

  // Statistik
  protected long _cons=0;
  protected long _commands=0;
  protected long _conTime=0;
  protected long _adminCommands=0;
  protected long _adminTries=0;
  protected long _adminFails=0;
  protected long _syncs=0;
  
  public DataServer(DBConnection con, Node configNode, Updateable updateMaster, StatisticWriter statMaster, int queryPort)
  {
    _con=con;
    _updateMaster=updateMaster;
    _statisticMaster=statMaster;
    _queryPort=queryPort;
    
    // Config einlesen
    NamedNodeMap nnl=configNode.getAttributes();
    Node attr=nnl.getNamedItem("name");
    if (null!=attr)
    {
      _name=attr.getNodeValue();
    }
    attr=nnl.getNamedItem("id");
    if (null!=attr)
    {
      String s=attr.getNodeValue();
      _id=Integer.parseInt(s);
      if (0==_id)
      {
        throw new IllegalArgumentException("id for dataserver must not be 0!");
      }
    }
    
    NodeList nl=configNode.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node node=nl.item(i);
      if (node.getNodeType()!=node.ELEMENT_NODE)
        continue;
      String nn=node.getNodeName();
      if (nn.equals("port"))
      {
        _port=Integer.parseInt(de.hendriklipka.ldse.common.Util.getTextData(node));
      }
      else if (nn.equals("address"))
      {
        _server=de.hendriklipka.ldse.common.Util.getTextData(node);
      }
      else if (nn.equals("password"))
      {
        _passwd=de.hendriklipka.ldse.common.Util.getTextData(node);
      }
      else if (nn.equals("parent"))
      {
        NodeList cnl=node.getChildNodes();
        for (int j=0;j<cnl.getLength();j++)
        {
          Node n=cnl.item(j);
          if (n.getNodeType()!=node.ELEMENT_NODE)
            continue;
          nn=n.getNodeName();
          if (nn.equals("port"))
          {
            _parentPort=Integer.parseInt(de.hendriklipka.ldse.common.Util.getTextData(n));
            Log.log("dataserver","config",1,"parent server on port "+_parentPort);
          }
          else if (nn.equals("address"))
          {
            _parentServer=de.hendriklipka.ldse.common.Util.getTextData(n);
            Log.log("dataserver","config",1,"parent server on address "+_parentServer);
          }
        }
      }
    }
  }
  
  public void run()
  {
    try
    {
      ServerSocket socket=null;
      if (null==_server)
      {
        socket=new ServerSocket(_port);
      }
      else
      {
        socket=new ServerSocket(_port,100,InetAddress.getByName(_server));
      }
      socket.setSoTimeout(1000);
      Log.log("dataserver","config",2,"awaiting connections");
      while (true)
      {
        try
        {
          // warten auf Verbindungen
          Socket s=socket.accept();
          SocketThread st=new SocketThread(s);
          Log.log("dataserver","net",1,"thread for "+s.getInetAddress().getHostAddress()+":"+s.getPort()+" local: "+s.getLocalPort());
          st.start();
          _cons++;
          st=null;
        }
        catch (IOException ex) {}
        if (_reconfig)
        {
          Log.log("dataserver","config",1,"doing reconfiguration");
          _reconfig=false;
          ServerSocket oldServer=socket;
          try
          {
            if (null==_server)
            {
              socket=new ServerSocket(_port);
            }
            else
            {
              socket=new ServerSocket(_port,100,InetAddress.getByName(_server));
            }
            socket.setSoTimeout(1000);
          }
          catch (Exception ex)
          {
            socket=oldServer;
          }
          if (socket!=oldServer)
          {
            oldServer.close();
          }
        }
      }
    }
    catch (IOException ex) 
    {
      ex.printStackTrace(Log.getWriter("dataserver","error",0));
    }
  }

  // Aufruffunktionen fuer die Datenbank
  public boolean existDocument(String name)
  {
    Log.log("dataserver","query",1,"DataServer.existDocument");
    return _con.existDocument(name);
  }
  
  public String unmarkDocument(String name)
  {
    _dataChanged=true;
    Log.log("dataserver","query",1,"DataServer.unmarkDocument");
    return _con.unmarkDocument(name);
  }
  
  public String putDocument(String name, DocumentDescriptor dd, WordDocument[] words)
  {
    _dataChanged=true;
    Log.log("dataserver","query",1,"DataServer.putDocument");
    return _con.putDocument(name,dd, words);
  }
  
  public DocumentDescriptor getDocumentDescriptor(String name)
  {
    Log.log("dataserver","query",1,"DataServer.getDocumentDescriptor");
    return _con.getDocumentDescriptor(name);
  }

  public String deleteDocument(String name)
  {
    _dataChanged=true;
    return _con.delete(name);
  }
  
  public String markSpider(int id)
  {
    return _con.markSpider(id);
  }
  
  public String sweepSpider(int id)
  {
    _dataChanged=true;
    return _con.sweepSpider(id);
  }
  
  public String deleteSpider(int id)
  {
    _dataChanged=true;
    return _con.deleteSpider(id);
  }
  
  public String[] getAllForSpider(int id)
  {
    return _con.getAllForSpider(id);
  }
  
  public String reconfig()
  {
    _updateMaster.update(null);
    return null;
  }
  
  public String putServerDescriptor(ServerDescriptor sd)
  {
    Log.log("dataserver","query",1,"DataServer.putServerDescriptor");
    _subServerChanged=true;
    return _con.putServerDescriptor(sd);
  }
  
  public String deleteSubServer(String name)
  {
    _subServerChanged=true;
    return _con.deleteSubServer(name);
  }
  
  public String deleteSubServer(int id)
  {
    _subServerChanged=true;
    return _con.deleteSubServer(id);
  }
  
  public String cleanDatabase()
  {
    _subServerChanged=true;
    _dataChanged=true;
    return _con.cleanDatabase();
  }
  

  public String writeStatistic(String fname)
  {
    try
    {
      if (null==fname)
        fname="";
      if(0==fname.trim().length())
      {
        fname="statistic.txt";
      }
      PrintWriter pw=new PrintWriter(new FileWriter(fname));
      _statisticMaster.writeStatistic(pw);
      pw.close();
      return null;
    }
    catch (IOException ex)
    {
      return ex.toString();
    }
  }

  // innere Klasse, die eine Verbindung handhabt
  class SocketThread extends Thread
  {
    protected Socket _socket=null;
    protected InputStream _istream=null;
    protected OutputStream _ostream=null;
    protected PrintWriter _writer=null;
    protected BufferedReader _reader=null;
    
    public SocketThread(Socket socket)
    {
      _socket=socket;
      try
      {
        _socket.setTcpNoDelay(true);
        _istream=_socket.getInputStream();
        _ostream=_socket.getOutputStream();
        _reader=new BufferedReader(new InputStreamReader(_istream),1);
        _writer=new PrintWriter(new OutputStreamWriter(_ostream));
      }
      catch (Exception ex)
      {
      }
    }
    
    public void run()
    {
      synchronized(_countLock)
      {
        _threadCount++;
      }
      long startTime=System.currentTimeMillis();
      try
      {
        // Solange, bis die verbindung zu ist
        while (true) // wird beendet mit IOException... oder mit "close"
        {
          // Kommando lesen
          String line=_reader.readLine();
          _commands++;
          Log.log("dataserver","query",1,"in: "+line);
          try
          {
            // und schauen, was es ist...
            if (line.startsWith("exist<>"))
            {
                // URL extrahieren
                line=line.substring(7);
                // Abfrage und Ergebnis senden
                if (existDocument(line))
                {
                _writer.println("yes");
                Log.log("dataserver","query",2,"yes");
                _writer.flush();
                }
                else
                {
                _writer.println("no");
                Log.log("dataserver","query",2,"no");
                _writer.flush();
                }
            }
            else if (line.startsWith("unmark<>"))
            {
                // URL extrahieren
                line=line.substring(8);
                // Abfrage und Ergebnis senden
                String s=unmarkDocument(line);
                if (null==s)
                {
                _writer.println("ok");
                _writer.flush();
                }
                else
                {
                _writer.println("error="+s);
                _writer.flush();
                }
            }
            else if (line.equals("close"))
            {
                Log.log("dataserver","query",1,"closing");
                break;
            }
            else if (line.startsWith("get<>"))
            {
                Log.log("dataserver","query",1,"get");
                // URL extrahieren
                line=line.substring(5);
                // Anfrage stellen 
                Log.log("dataserver","query",1,"get: "+line);
                Properties props=getDocumentDescriptor(line).getData();
                if (null==props)
                {
                Log.log("dataserver","query",1,"no document properties");
                _writer.println("error=no document properties");
                _writer.flush();
                continue;
                }
                // Daten schreiben
                Enumeration keys=props.keys();
                Log.log("dataserver","query",1,"writing data");
                _writer.println("ok");
                while (keys.hasMoreElements())
                {
                String key=(String)keys.nextElement();
                String val=props.getProperty(key);
                _writer.println(key+"="+val);
                _writer.flush();
                }
                _writer.println("<><>");
                _writer.flush();
                Log.log("dataserver","query",1,"get: finished");
            }
            else if (line.startsWith("put<>"))
            {
                try
                {
                    // URL extrahieren
                    line=line.substring(5);
                    Log.log("dataserver","query",2,"name="+line);
                    // DocumentDescriptor lesen
                    Properties props=new Properties();
                    while (true)
                    {
                        String answer=_reader.readLine();
                        Log.log("dataserver","query",4,"line="+answer);
                        if (null==answer)
                            break;
                        if (answer.equals("<>"))
                            break;
                        int pos=answer.indexOf("=");
                        if (-1==pos)
                            continue;
                        String name=answer.substring(0,pos);
                        String val=answer.substring(pos+1);
                        Log.log("dataserver","query",5,"got property: "+name+"="+val);
                        props.put(name,val);
                    }
                    Log.log("dataserver","query",3,"got document descriptor");
                    // Triplets extrahieren
                    TripletList tl=new TripletList();
                    while (true)
                    {
                        String answer=_reader.readLine();
                        Log.log("dataserver","query",4,"line="+answer);
                        if (null==answer)
                            break;
                        if (answer.equals("<>"))
                            break;
                        int pos=answer.indexOf("=");
                        if (-1==pos)
                            continue;
                        String num=answer.substring(0,pos);
                        String count=answer.substring(pos+1);
                        int n=Integer.parseInt(num);
                        int c=Integer.parseInt(count);
                        Triplet tr=new Triplet(n,c);
                        tl.addTriplet(tr);
                    }
                    
                    // Worte einlesen
                    Vector v=new Vector(100,100);
                    while (true)
                    {
                        String answer=_reader.readLine();
                        Log.log("dataserver","query",4,"line="+answer);
                        if (null==answer)
                            break;
                        if (answer.equals("<><>"))
                            break;
                        int pos=answer.indexOf("=");
                        if (-1==pos)
                            continue;
                        String word=answer.substring(0,pos);
                        answer=answer.substring(pos+1);
                        pos=answer.indexOf("=");
                        if (-1==pos)
                            continue;
                        String count=answer.substring(0,pos);
                        WordDocument wd=new WordDocument();
                        wd._count=Integer.parseInt(count);
                        WordData wd2=new WordData();
                        wd2._word=word;
                        wd._word=wd2;
                        answer=answer.substring(pos+1);
                        StringTokenizer st=new StringTokenizer(answer,",");
                        TripletList list=new TripletList();
                        while (st.hasMoreElements())
                        {
                            String s=(String)st.nextElement();
                            int n=Integer.parseInt(s);
                            Triplet tr=new Triplet(n,1);
                            list.addTriplet(tr);
                        }
                        wd2._triplets=list;
                        v.addElement(wd);
                    }
                    WordDocument[] wda=new WordDocument[v.size()];
                    for (int i=0;i<wda.length;i++)
                    {
                        wda[i]=(WordDocument)v.elementAt(i);
                    }
                    DocumentDescriptor dd=new DocumentDescriptor(props);
                    dd.setList(tl);
                    // Original
                    /*
                    if (putDocument(line,dd))
                    {
                    _writer.println("ok");
                    _writer.flush();
                    }
                    else
                    {
                    _writer.println("error0");
                    _writer.flush();
                    }
                    */
                    // Performance improvement
                    // so kann der Client schon weitermachen, waehrend das Dokument noch geschrieben wird
                    // allerdings entfaellt die Fehlerbehandlung
                    _writer.println("ok");
                    _writer.flush();
                    putDocument(line,dd,wda);
                }
                catch (Throwable ex)
                {
                    ex.printStackTrace();
                }
            }
            else if (line.startsWith("sget<>"))
            {
                // ID extrahieren
                line=line.substring(6);
                int id=Integer.parseInt(line);
                // Abfrage und Ergebnis senden
                String[] sa=getAllForSpider(id);
                if (null==sa)
                {
                _writer.write("error=no documents found\n");
                _writer.flush();
                Log.log("dataserver","query",1,"sget: no documents");
                continue;
                }
                // Ergebnis schreiben
                for (int i=0;i<sa.length;i++)
                {
                _writer.println(sa[i]);
                _writer.flush();
                }
                _writer.println("<><>\n");
                _writer.flush();
                Log.log("dataserver","query",1,"sget: found "+sa.length+" documents");
            }
            // Delete Document
            else if (line.startsWith("deld<>"))
            {
                _adminCommands++;
                if (!_isAdmin)
                {
                _writer.println("error=not admin");
                _writer.flush();
                continue;
                }
                // URL extrahieren
                line=line.substring(6);
                // Abfrage und Ergebnis senden
                String s=deleteDocument(line);
                if (null==s)
                {
                _writer.println("ok");
                Log.log("dataserver","query",2,"ok");
                _writer.flush();
                }
                else
                {
                _writer.println("error="+s);
                Log.log("dataserver","query",2,"error="+s);
                _writer.flush();
                }
            }
            // Delete Spider
            else if (line.startsWith("dels<>"))
            {
                _adminCommands++;
                if (!_isAdmin)
                {
                _writer.println("error=not admin");
                _writer.flush();
                continue;
                }
                // ID extrahieren
                line=line.substring(6);
                try
                {
                int id=Integer.parseInt(line);
                String s=deleteSpider(id);
                if (null==s)
                {
                    _writer.println("ok");
                    _writer.flush();
                }
                else
                {
                    _writer.println("error="+s);
                    _writer.flush();
                }
                }
                catch(NumberFormatException ex)
                {
                _writer.println("error="+ex.toString());
                _writer.flush();
                }
            }
            // Delete SubServer (Broker)
            else if (line.startsWith("delb<>"))
            {
                _adminCommands++;
                if (!_isAdmin)
                {
                _writer.println("error=not admin");
                _writer.flush();
                continue;
                }
                // ID extrahieren
                line=line.substring(6);
                try
                {
                int id=Integer.parseInt(line);
                String s=deleteSubServer(id);
                if (null==s)
                {
                    _writer.println("ok");
                    _writer.flush();
                }
                else
                {
                    _writer.println("error="+s);
                    _writer.flush();
                }
                }
                catch(NumberFormatException ex)
                {
                String s=deleteSubServer(line);
                if (null==s)
                {
                    _writer.println("ok");
                    _writer.flush();
                }
                else
                {
                    _writer.println("error="+s);
                    _writer.flush();
                }
                }
                _writer.flush();
            }          
            else if (line.startsWith("mark<>"))
            {
                // ID extrahieren
                line=line.substring(6);
                int id=Integer.parseInt(line);
                // Abfrage und Ergebnis senden
                String s=markSpider(id);
                if (null==s)
                {
                _writer.println("ok");
                Log.log("dataserver","query",2,"ok");
                _writer.flush();
                }
                else
                {
                _writer.println("error="+s);
                Log.log("dataserver","query",2,"error="+s);
                _writer.flush();
                }
            }
            else if (line.startsWith("sweep<>"))
            {
                // ID extrahieren
                line=line.substring(7);
                int id=Integer.parseInt(line);
                // Abfrage und Ergebnis senden
                String s=sweepSpider(id);
                if (null==s)
                {
                _writer.println("ok");
                Log.log("dataserver","query",2,"ok");
                _writer.flush();
                }
                else
                {
                _writer.println("error="+s);
                Log.log("dataserver","query",2,"error="+s);
                _writer.flush();
                }
            }
            else if (line.startsWith("admin<>"))
            {
                _adminTries++;
                line=line.substring(7);
                if (line.equals(_passwd))
                {
                _isAdmin=true;
                _writer.println("ok");
                }
                else
                {
                _isAdmin=false;
                _adminFails++;
                _writer.println("error=wrong password");
                }
                Log.log("dataserver","query",0,"isAdmin="+(_isAdmin?(new String("true")):(new String("false"))));
                _writer.flush();
            }
            else if (line.equals("shutdown"))
            {
                _adminCommands++;
                Log.log("dataserver","query",0,"shutting down");
                if (!_isAdmin)
                {
                _writer.println("error=not admin");
                _writer.flush();
                continue;
                }
                if (null==_con)
                {
                _writer.println("ok");
                }
                else
                {
                String s=_con.shutdown();
                if(null==s)
                {
                    _writer.println("ok");
                    _con=null;
                }
                else
                {
                    _writer.println("error="+s);
                }
                }
                _writer.flush();
                _socket.close();
                System.exit(1);
            }
            else if (line.equals("reconfig"))
            {
                _adminCommands++;
                if (!_isAdmin)
                {
                _writer.println("error=not admin");
                _writer.flush();
                continue;
                }
                String s=reconfig();
                if(null==s)
                {
                _writer.println("ok");
                }
                else
                {
                _writer.println("error="+s);
                }
                _writer.flush();
                Log.log("dataserver","query",1,"finished reconfiguration");
            }
            else if (line.startsWith("wrst<>"))
            {
                _adminCommands++;
                if (!_isAdmin)
                {
                _writer.println("error=not admin");
                _writer.flush();
                continue;
                }
                // ID extrahieren
                line=line.substring(6);
                String s=writeStatistic(line);
                if(null==s)
                {
                _writer.println("ok");
                _con=null;
                }
                else
                {
                _writer.println("error="+s);
                }
                _writer.flush();
            }
            else if (line.equals("clen"))
            {
                _adminCommands++;
                if (!_isAdmin)
                {
                _writer.println("error=not admin");
                _writer.flush();
                continue;
                }
                String s=cleanDatabase();
                if(null==s)
                {
                _writer.println("ok");
                _con=null;
                }
                else
                {
                _writer.println("error="+s);
                }
            }
            else if (line.startsWith("puts<>"))
            { 
                line=line.substring(6); // ID
                int id=Integer.parseInt(line);
                Log.log("dataserver","query",1,"puts="+id);
                // ServerDescriptor lesen
                Properties props=new Properties();
                while (true)
                {
                String answer=_reader.readLine();
                if (answer.equals("<>"))
                    break;
                int pos=answer.indexOf("=");
                if (-1==pos)
                    continue;
                String name=answer.substring(0,pos);
                String val=answer.substring(pos+1);
                props.put(name,val);
                }
                Log.log("dataserver","query",1,"puts: got all properties");
                ServerDescriptor sd=new ServerDescriptor(props,id);
                // Triplets und Zusatzdaten extrahieren
                while (true)
                {
                String answer=_reader.readLine();
                if (answer.equals("<><>"))
                    break;
                int pos=answer.indexOf('=');
                if (-1==pos)
                    continue;
                String num=answer.substring(0,pos);
                answer=answer.substring(pos+1);
                pos=answer.indexOf('=');
                if (-1==pos)
                    continue;
                String count=answer.substring(0,pos);
                answer=answer.substring(pos+1);
                pos=answer.indexOf('=');
                if (-1==pos)
                    continue;
                String docCount=answer.substring(0,pos);
                String maxNum=answer.substring(pos+1);
                int n=Integer.parseInt(num);
                int c=Integer.parseInt(count);
                int mn=Integer.parseInt(maxNum);
                int dc=Integer.parseInt(docCount);
                sd.addTriplet(n,c,dc, mn);
                }
                Log.log("dataserver","query",1,"puts: got all triplets");
                String s=putServerDescriptor(sd);
                if (null==s)
                {
                _writer.println("ok");
                _writer.flush();
                }
                else
                {
                _writer.println("error="+s);
                _writer.flush();
                }
            }
            else
            {
                Log.log("dataserver","query",1,"unknown command: "+line);
                _commands--;
            }
            System.gc();
          }
          catch (Throwable ex)
          {
            ex.printStackTrace(Log.getWriter("dataserver","error",0));
          }
        }
      }
      catch (Throwable ex)
      {
        ex.printStackTrace(Log.getWriter("dataserver","error",0));
      }
      finally
      {
        long end=System.currentTimeMillis();
        _conTime+=(end-startTime);
        try
        {
          _reader.close();
          _writer.close();
          _istream.close();
          _ostream.close();
          _socket.close();
        }
        catch (IOException ex) {}
        synchronized(_countLock)
        {
          _threadCount--;
          ServerDescriptor sd=null;
          // nur wenn keine Verbindungen mehr offen sind, wird
          // synchronisiert
          if (0==_threadCount)
          {
            if(_dataChanged)
            {
              Log.log("dataserver","net",1,"creating local server descriptor");
              // der lokale Descriptor bekommt immer die 0 als ID
              sd=_con.createLocalServerDescriptor(0,_name,_queryPort);
              _con.putServerDescriptor(sd);
            }
            // Das holen des ServerDescriptor laeuft im Lock ab, da wir auf
            // der DB ohnehin einen ReaderLock haben und andere Verbindungen
            // damit ohnehin nichts modifizieren koennen
            if ((null!=_parentServer)&&(_subServerChanged||_dataChanged))
            {
              Log.log("dataserver","net",1,"creating server descriptor");
              int[] sda=_con.getSubServers();
              // Optimierung:
              // wenn es nur einen ServerDescriptor gibt und das der lokale Server
              // ist, brauchen wir den nicht neu zu erzeugen...
              if (1==sda.length)
              {
                if (0==sda[0])
                {
                  sd.setID(_id);
                }
                else
                {
                  sd=_con.createServerDescriptor(_id,_name,_queryPort);
                }
              }
              else
              {
                sd=_con.createServerDescriptor(_id,_name,_queryPort);
              }
            }
            else
            {
              sd=null;
            }
          } // synchronized
          // Synchronisation mit dem Master kann ausserhalb des Locks erfolgen
          // da die Daten hier bereits feststehen und die Funktion synchronized ist
          if (null!=sd)
          {
            synchronizeMaster(sd);
          }
        }
      }
    }
  }
  
  public void update(Node configNode)
  {
    boolean newData=false;
    int port=_port;
    String server=_server;
    String passwd=_passwd;
    Log.log("dataserver","config",1,"reading reconfiguration");
    NodeList nl=configNode.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node node=nl.item(i);
      if (node.getNodeType()!=node.ELEMENT_NODE)
        continue;
      String nn=node.getNodeName();
      if (nn.equals("port"))
      {
        _port=Integer.parseInt(de.hendriklipka.ldse.common.Util.getTextData(node));
        if (_port!=port)
          newData=true;
      }
      else if (nn.equals("address"))
      {
        _server=de.hendriklipka.ldse.common.Util.getTextData(node);
        if (_server!=server)
          newData=true;
      }
      else if (nn.equals("password"))
      {
        _passwd=de.hendriklipka.ldse.common.Util.getTextData(node);
        if (_passwd!=passwd)
          newData=true;
      }
    }
    _reconfig=newData;
    if (!newData)
    Log.log("dataserver","config",1,"no new config data found!");
  }
  
  public synchronized void synchronizeMaster(ServerDescriptor sd)
  {
    Log.log("dataserver","net",1,"synchronize with master");
    DataClient dc=new DataClient(_parentServer,_parentPort);
    dc.putServerDescriptor(sd);
    dc.close();
    _dataChanged=false;
    _subServerChanged=false;
    _syncs++;
  }
  
  public void writeStatistic(PrintWriter pw) throws IOException
  {
    pw.println("*** DataServer");
    pw.println("connections:            "+_cons);
    pw.println("commands:               "+_commands);
    pw.println("time in connections:    "+_conTime+" ms");
    pw.println("admin commands:         "+_adminCommands);
    pw.println("tries to be admin:      "+_adminTries);
    pw.println("fails to be admin:      "+_adminFails);
    pw.println("# of syncs with master: "+_syncs);
    pw.println();
  }
}

